@@ -12,7 +12,7 @@
 #
 ##############################################################################
 
-import os, re, shutil
+import os, re, shutil, sys
 import zc.buildout
 import zc.recipe.egg
 
@@ -49,6 +49,8 @@ class Recipe:
             zope2_bin = os.path.join(options['zope2-location'],
                                      'utilities')
         mkzopeinstance = os.path.join(zope2_bin, 'mkzopeinstance.py')
+        if sys.platform[:3].lower() == "win":
+            mkzopeinstance = '"%s"' % mkzopeinstance
 
         assert os.spawnl(
             os.P_WAIT, options['executable'], options['executable'],
@@ -170,7 +172,7 @@ class Recipe:
             # custom log 
             else:
                 event_log = custom_event_log
-            
+ 
             z_log_name = options.get('z2-log', os.path.sep.join(('log', self.name + '-Z2.log',)))
             z_log = os.path.join(var_dir, z_log_name)
             z_log_dir = os.path.dirname(z_log)
@@ -179,36 +181,42 @@ class Recipe:
             
             z_log_level = options.get('z2-log-level', 'WARN')
             
+            # access event log
+            custom_access_event_log = options.get('access-log-custom', None)
+            # filelog directive
+            if custom_access_event_log is None: 
+                access_event_log = access_event_logfile % {'z_log': z_log}
+            # custom directive
+            else:
+                access_event_log = custom_access_event_log
+
             default_zpublisher_encoding = options.get('default-zpublisher-encoding', 'utf-8')
             if default_zpublisher_encoding:
                 default_zpublisher_encoding = 'default-zpublisher-encoding %s' % default_zpublisher_encoding
             
-            relstorage = options.get('rel-storage', None)
+            relstorage = options.get('rel-storage')
             if relstorage is not None:
-                rel_storage = {'dbname': 'test', 'host': 'localhost',
-                               'user': 'test', 'password': 'test'}
-                def _extract(el):
-                    if el.strip('') == '':
-                        return None
-                    el = el.split(' ')
-                    if len(el) != 2:
-                        return None
-                    return el[0].strip(), el[1].strip()
+                def _split(el):
+                    el = el.strip().split()
+                    return len(el) == 2 and el or None
 
-                elements = [_extract(el) for el in relstorage.split('\n') 
-                            if _extract(el) is not None]
+                rel_storage = dict(
+                    _split(el) for el in relstorage.splitlines() 
+                    if _split(el) is not None)
+                type_ = rel_storage.pop('type', 'postgresql')
                 
-                type_ = 'postgresql' 
-                for key, value in elements:
-                    if key == 'type':
-                        type_ = value
-                    elif key in rel_storage:
-                        rel_storage[key] = value
+                if type_ == 'postgresql' and not 'dsn' in rel_storage:
+                    # Support zope2instance 1.4 style interpolation for
+                    # postgresql
+                    template = ("dbname='%(dbname)s' user='%(user)s' "
+                                "host='%(host)s' password='%(password)s'")
+                    rel_storage = dict(dsn=template % rel_storage)                    
 
-                if type_ == 'postgresql': 
-                    storage_snippet = rel_storage_postgres_template % rel_storage
-                else:
-                    storage_snippet = rel_storage_oracle_template % rel_storage
+                opts = dict(
+                    type=type_,
+                    opts='\n'.join(' ' * 12 + ' '.join(item)
+                                   for item in rel_storage.iteritems()))
+                storage_snippet = rel_storage_template % opts
 
             else:
                 file_storage = options.get('file-storage', 
@@ -242,9 +250,12 @@ class Recipe:
                 zeo_client_name = options.get('zeo-client-name', self.name)
                 zeo_var_dir = options.get('zeo-var',
                                           os.path.join(instance_home, 'var'))
+                zeo_client_client = options.get('zeo-client-client', '')
                 shared_blob_dir = options.get('shared-blob', 'no')
                 if zeo_client_name:
                     zeo_client_name = 'zeo-client-name %s' % zeo_client_name
+                if zeo_client_client:
+                    zeo_client_client = 'client %s' % zeo_client_client
                 if blob_storage:
                     storage_snippet_template = zeo_blob_storage_template
                 else:
@@ -254,11 +265,13 @@ class Recipe:
                     shared_blob_dir = shared_blob_dir,
                     zeo_address = zeo_address,
                     zeo_client_cache_size = zeo_client_cache_size,
+                    zeo_client_client = zeo_client_client,
                     zeo_storage = zeo_storage,
                     zeo_var_dir=zeo_var_dir,
                     )
             else:
-                # no zeo-client, so no zeo_client_name
+                # no zeo-client
+                zeo_client_client = ''
                 zeo_client_name = ''
 
             template = zope_conf_template
@@ -287,7 +300,7 @@ class Recipe:
                                         ip_address = ip_address,
                                         event_log = event_log,
                                         event_log_level = event_log_level,
-                                        z_log = z_log,
+                                        access_event_log = access_event_log,
                                         z_log_level = z_log_level,
                                         default_zpublisher_encoding = default_zpublisher_encoding,
                                         storage_snippet = storage_snippet.strip(),
@@ -482,24 +495,13 @@ file_storage_template="""
       path %s
     </filestorage>
 """
-rel_storage_postgres_template="""
-    %%import relstorage
-    <relstorage>
-        <postgresql>
-            # The dsn is optional, as are each of the parameters in the dsn.
-            dsn dbname='%(dbname)s' user='%(user)s' host='%(host)s' password='%(password)s'
-        </postgresql>
-    </relstorage>
-"""
 
-rel_storage_oracle_template="""
+rel_storage_template="""
     %%import relstorage
     <relstorage>
-        <oracle>
-            user %(user)s
-            password %(pass)s
-            dsn %(dns)s
-        </oracle>
+        <%(type)s>
+%(opts)s
+        </%(type)s>
     </relstorage>
 """
 blob_storage_template="""
@@ -520,6 +522,7 @@ zeo_storage_template="""
       name zeostorage
       var %(zeo_var_dir)s
       cache-size %(zeo_client_cache_size)s
+      %(zeo_client_client)s
     </zeoclient>
 """.strip()
 
@@ -533,6 +536,7 @@ zeo_blob_storage_template="""
       name zeostorage
       var %(zeo_var_dir)s
       cache-size %(zeo_client_cache_size)s
+      %(zeo_client_client)s
     </zeoclient>
 """.strip()
 
@@ -543,6 +547,13 @@ event_logfile = """
   </logfile>
 """.strip()
 
+access_event_logfile = """
+  <logfile>
+    path %(z_log)s
+    format %%(message)s
+  </logfile>
+""".strip()
+
 # The template used to build zope.conf
 zope_conf_template="""\
 instancehome %(instance_home)s
@@ -565,10 +576,7 @@ verbose-security %(verbose_security)s
 
 <logger access>
   level %(z_log_level)s
-  <logfile>
-    path %(z_log)s
-    format %%(message)s
-  </logfile>
+  %(access_event_log)s
 </logger>
 
 <http-server>
